import processing.core.*; 
import processing.xml.*; 

import oscP5.*; 
import netP5.*; 

import java.applet.*; 
import java.awt.*; 
import java.awt.image.*; 
import java.awt.event.*; 
import java.io.*; 
import java.net.*; 
import java.text.*; 
import java.util.*; 
import java.util.zip.*; 
import java.util.regex.*; 

public class Confetti02 extends PApplet {




OscP5 oscP5;
NetAddress myRemoteLocation;

int [][] matrix = new int[0][0];
int [][] volumns = new int[0][0];
int [] current = new int[2];
boolean upVal = true;
int offset = 30;
int big = 30;

int rows = 32;
int columns = 10;


public void setup(){
  size(1000,580);
  cursor(CROSS);
  PFont f = createFont("Verdana",9);
  textFont(f);
  oscP5 = new OscP5(this,12000);
  myRemoteLocation = new NetAddress("127.0.0.1",12000);
  prepareMatrix();
}

public void draw(){
  background(125);
  paintMatrix();
}


public void paintMatrix(){
  stroke(0xffD4CEE5);
  for(int i=0;i<matrix.length;i++){
    for(int j=0;j<matrix[i].length;j++){
      int now = matrix[i][j];
      int vol = volumns[i][j];
      int x = i*big+offset;
      int y = j*big+offset;
      if(now == 0){        
        fill(120,120);
      }
      else if(now >= 1){   
        fill(0,vol,now);
      }
      rect(x,y,big-1,big-1);  
      
      fill(0,0,now,120);      
      rect(x,y,now,vol);  
      fill(255);
      if(now >= 1){
        text(now, x+5,y+10);
      }
      if(vol >= 1)
        text(vol, x+5,y+25);
    }
  }
  text(nuVal,width/2,30);  
  if(upVal){
    fill(120,0,0);
    rect(width/2-20,10,10,10);  
  }else{
    fill(0,0,120);
    rect(width/2-20,20,10,10);    
  }  
}


public void prepareMatrix(){
  // initialize rows:
  for(int i=0;i<rows;i++){
    //print(i);
    matrix = (int[][])append(matrix,new int[0]);
    matrix[i] = new int[0];
    volumns = (int[][])append(volumns,new int[0]);
    volumns[i] = new int[0];
  }

  // initialize columns:
  for(int i=0;i<matrix.length;i++){
    for(int j=0;j<columns;j++){
      matrix[i] = append(matrix[i],0); //int(random(0,255)));
      volumns[i] = append(volumns[i],0);
    }
  }  
}


public void mousePressed(){
  int i = PApplet.parseInt(floor((mouseX-offset)/PApplet.parseFloat(big)));
  int j = PApplet.parseInt(floor((mouseY-offset)/PApplet.parseFloat(big)));
//  println(i + "  " + j);
  if(i < 0 || j < 0 || i >= matrix.length || j >= matrix[0].length)
    return;
  current[0] = i;
  current[1] = j;
  if(matrix[i][j] >= 1){
    matrix[i][j] = 0;
    volumns[i][j] = 0;
    note(i,j,0,0);
  }else{
    matrix[i][j] = 50;
    volumns[i][j] = 50;    
    note(i,j,50,50);
  }
}


public void mouseDragged(){
  int i = current[0];
  int j = current[1];

  fill(0,20);
  int x = i*big+offset;
  int y = j*big+offset;
  int w = mouseX-x;
  int h = mouseY-y;
  if(w > 0 && h > 0){
    rect(x,y,w,h);
    matrix[i][j] = w;
    volumns[i][j] = h;
    note(i, j, w, h);
  }
}


String nuVal = "";

public void keyPressed(){
  int i = current[0];
  int j = current[1];

  if(keyCode == UP)
    volumns[i][j]--;
  else if(keyCode == DOWN)  
    volumns[i][j]++;
  else if(keyCode == LEFT){
    if(matrix[i][j]>0)
      matrix[i][j]--;
  }
  else if(keyCode == RIGHT){
    matrix[i][j]++;
  }
  else if(key >= '0' && key <= '9'){// && key <= '9'){
    nuVal += key;
  }else if(key == TAB){
    upVal = !upVal;    
  }else if(key == BACKSPACE){
    if(nuVal.length()-1 >= 0)
      nuVal = nuVal.substring(0,nuVal.length()-1);    
  }else if(key == RETURN || key == ENTER){
    int[][] tmp;
    if(upVal)
      tmp = matrix;
    else
      tmp = volumns;
    tmp[i][j] = PApplet.parseInt(nuVal);
    nuVal = "";
  }
  else if(key == 'x'){
    clearAll();
  }else if(key == 's'){
    saveAll();
  }else if(key == 'o'){
    loadAll();
  }
  note(i, j, matrix[i][j], volumns[i][j]);  
}


public void clearAll(){
  for(int n=0;n<matrix.length;n++){
      for(int m=0;m<matrix[n].length;m++){
        matrix[n][m] = volumns[n][m] = 0;
        note(n, m, 0,0);
      }
    }
}

public void saveAll(){
  String saveAs = selectOutput();
  if (saveAs == null)
    return;
  
  String[] output = new String[0];
  for(int n=0;n<matrix.length;n++){
    for(int m=0;m<matrix[n].length;m++){
      if(matrix[n][m] > 0 &&  volumns[n][m]> 0){
        String tmp = n+","+m+","+matrix[n][m]+","+volumns[n][m];
        output = append(output,tmp);
      }
    }
  }
  saveStrings(saveAs+".txt",output);
}


public void loadAll(){
  String loadPath = selectInput();
  if (loadPath == null) {
    ; ; ; ; //println("No file was selected...");
    return;
  }
  clearAll();
  
  String lines[] = loadStrings(loadPath);
  for (int i=0; i < lines.length; i++) {
    String[] o = split(lines[i],",");
    note(PApplet.parseInt(o[0]),PApplet.parseInt(o[1]),PApplet.parseInt(o[2]),PApplet.parseInt(o[3]));
    matrix[PApplet.parseInt(o[0])][PApplet.parseInt(o[1])] = PApplet.parseInt(o[2]);;  
    volumns[PApplet.parseInt(o[0])][PApplet.parseInt(o[1])] = PApplet.parseInt(o[3]);;      
  }  
}


public void note(int x, int y, int pitch, int vol){
  int i = y * rows + x;
//      value = value/100.0;
  OscMessage myMessage = new OscMessage("/note");
  myMessage.add(pitch);
  myMessage.add(vol);
  myMessage.add(i);      
  oscP5.send(myMessage, myRemoteLocation);   
}



  static public void main(String args[]) {
    PApplet.main(new String[] { "--bgcolor=#C0C0C0", "Confetti02" });
  }
}
